using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;
using System.Diagnostics.CodeAnalysis;
using System.Runtime.InteropServices;
using System.Security.Permissions;
using System.Globalization;

namespace FimExtensions.FimActivityLibrary
{
    /// <summary>
    /// Provides a set of methods and properties that enable applications to trace the execution of code and associate trace messages with their source. 
    /// </summary>
    [ComVisible(false)]
    [SuppressMessage("Microsoft.Naming", "CA1711:IdentifiersShouldNotHaveIncorrectSuffix")]
    internal class TraceSourceEx : TraceSource
    {
        /// <summary>
        /// The instance for trace within the common dll
        /// </summary>
        internal static TraceSourceEx Instance
        {
            get { return new TraceSourceEx("FimExtensions.FimActivityLibrary"); }
        }

        /// <summary>
        /// Const format string.
        /// </summary>
        private const string FormatString = "{0:d} {0:T} -- {1}";

        private const int MaxLength = 32766;

        /// <summary>
        /// Initializes a new instance of the TraceSource class, using the specified name for the source and the default source level at which tracing is to occur.
        /// </summary>
        /// <param name="name">The name of the source, typically the name of the application.</param>
        public TraceSourceEx(string name) : base(name) { }

        /// <summary>
        /// Writes a trace event at SourceLevels.Critical (0x0001) using the specified message.
        /// </summary>
        /// <param name="message">The informative message to write.</param>
        [Conditional("TRACE")]
        public void TraceFatal(string message)
        {
            TraceEvent(TraceEventType.Critical, 0, FormatMessage(FormatString, MaxLength, DateTime.UtcNow, message));
        }

        /// <summary>
        /// Writes a trace event at SourceLevels.Error (0x0003)  using the specified message.
        /// </summary>
        /// <param name="message">The informative message to write.</param>
        [Conditional("TRACE")]
        public void TraceError(string message)
        {
            TraceEvent(TraceEventType.Error, 0, FormatMessage(FormatString, MaxLength, DateTime.UtcNow, message));
        }

        /// <summary>
        /// Writes a trace event at SourceLevels.Warning (0x0007)  using the specified message.
        /// </summary>
        /// <param name="message">The informative message to write.</param>
        [Conditional("TRACE")]
        public void TraceWarning(string message)
        {
            TraceEvent(TraceEventType.Warning, 0, FormatMessage(FormatString, MaxLength, DateTime.UtcNow, message));
        }

        /// <summary>
        /// Writes a trace event at SourceLevels.Information (0x000F) using the specified message.
        /// </summary>
        /// <param name="message">The informative message to write.</param>
        [Conditional("TRACE")]
        public void TraceInfo(string message)
        {
            TraceEvent(TraceEventType.Information, 0, FormatMessage(FormatString, MaxLength, DateTime.UtcNow, message));
        }

        /// <summary>
        /// Writes a trace event at SourceLevels.Verbose (0x001F) using the specified message.
        /// </summary>
        /// <param name="message">The informative message to write.</param>
        [Conditional("TRACE")]
        public void TraceVerbose(string message)
        {
            TraceEvent(TraceEventType.Verbose, 0, FormatMessage(FormatString, MaxLength, DateTime.UtcNow, message));
        }

        /// <summary>
        /// Writes a start event using the specified message
        /// </summary>
        /// <param name="message">The informative message to write.</param>
        /// 
        [EnvironmentPermissionAttribute(SecurityAction.LinkDemand, Unrestricted = true)]
        [Conditional("TRACE")]
        public void TraceStart(String message)
        {
            TraceEvent(TraceEventType.Start, 0, FormatMessage(FormatString, MaxLength, DateTime.UtcNow, message));
            Trace.CorrelationManager.StartLogicalOperation();
        }

        /// <summary>
        /// Writes a start event using the specified message
        /// </summary>
        /// <param name="id">A numeric identifier for the event.</param>
        /// <param name="message">The informative message to write.</param>
        /// 
        [EnvironmentPermissionAttribute(SecurityAction.LinkDemand, Unrestricted = true)]
        [Conditional("TRACE")]
        public void TraceStart(int id, String message)
        {
            TraceEvent(TraceEventType.Start, id, FormatMessage(FormatString, MaxLength, DateTime.UtcNow, message));
            Trace.CorrelationManager.StartLogicalOperation();
        }

        /// <summary>
        /// Writes a stop event using the specified message
        /// </summary>
        /// <param name="message">The informative message to write.</param>
        /// 
        [EnvironmentPermissionAttribute(SecurityAction.LinkDemand, Unrestricted = true)]
        [Conditional("TRACE")]
        public void TraceStop(String message)
        {
            TraceEvent(TraceEventType.Stop, 0, FormatMessage(FormatString, MaxLength, DateTime.UtcNow, message));
            Trace.CorrelationManager.StopLogicalOperation();
        }

        /// <summary>
        /// Writes a stop event using the specified message
        /// </summary>
        /// <param name="id">A numeric identifier for the event.</param>
        /// <param name="message">The informative message to write.</param>
        /// 
        [EnvironmentPermissionAttribute(SecurityAction.LinkDemand, Unrestricted = true)]
        [Conditional("TRACE")]
        public void TraceStop(int id, String message)
        {
            TraceEvent(TraceEventType.Stop, id, FormatMessage(FormatString, MaxLength, DateTime.UtcNow, message));
            Trace.CorrelationManager.StopLogicalOperation();
        }

        static string FormatMessage(string format, int maxLength, params object[] args)
        {
            string formattedString = String.Format(format, args);

            if (formattedString.Length > maxLength)
            {
                formattedString = formattedString.Substring(0, maxLength - 3);
                formattedString += "...";
            }

            return formattedString;
        }
    }
}